#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: __init__.py 
#       作者  : Qazse 
#       时间  : 2019/4/13
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------
import sys as _sys
import io
from qazse import file
from qazse import log
from qazse import requser
from qazse import seo
from qazse import text
from qazse import time
from qazse import file
from qazse import sys
from qazse import spiders
from qazse import db


_sys.stdout = io.TextIOWrapper(_sys.stdout.buffer, encoding='gb18030')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: db 
#       作者  : Qazse 
#       时间  : 2019/4/18
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------
import redis

class redis_db():

    def __init__(self,host='192.168.1.10',port=6379,db=0,password=''):
        self.r = redis.Redis(host=host, port=port, db=db,password=password)


    def list_to_set(self,list,name='discharge'):
        """
        写list 到 set
        :param list:
        :param name:
        :return:
        """
        for data in list:
            self.r.sadd(name,data)

    def val_to_set(self,val,name='discharge'):
        """
        写值到set
        :param val:
        :param name:
        :return: 失败 返回 0
        """
        return self.r.sadd(name,val)


def export_db_to_model(host_prot, username, password, db, tables=None, outfile='models.py'):
    """
    导出数据库模型到文件
    :param host:
    :param username:
    :param password:
    :param tables:
    :param outfile:
    :return:
    """
    if tables:
        cmd = 'sqlacodegen --outfile=%s mysql://%s:%s@%s/%s --tables %s' %(outfile,username,password,host_prot,db,','.join(tables))
    else:
        cmd = 'sqlacodegen --outfile=%s mysql://%s:%s@%s/%s' % (outfile, username, password, host_prot, db)
    import os
    os.popen(cmd)


def mongodb_db(host=None,password=None):
    """
    返回一个mongodb存储对象
    :param mongo_db_link:链接地址
    :return:
    """
    from pymongo import MongoClient

    if not host:
        host = ['localhost:27017']
    return MongoClient(host=host)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: time 
#       作者  : Qazse 
#       时间  : 2019/4/13
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------
import time


def timestamp(timestamp=time.time(), thirteen=False):
    """
    获取时间戳
    :param timestamp: 时间戳
    :param thirteen: 取三十位
    :return:
    """
    if thirteen:
        return int(timestamp)
    else:
        return int(timestamp * 1000)


def date_to_timestamp(date, format_string="%Y-%m-%d %H:%M:%S"):
    """
    将时间转化为时间戳
    :param date:
    :param format_string:
    :return:
    """
    time_array = time.strptime(date, format_string)
    time_stamp = int(time.mktime(time_array))
    return time_stamp


def timestamp_to_date(time_stamp, format_string="%Y-%m-%d %H:%M:%S"):
    """
    将时间戳转换为时间
    :param time_stamp:
    :param format_string:
    :return:
    """
    time_array = time.localtime(time_stamp)
    str_date = time.strftime(format_string, time_array)
    return str_date


def now(get13 = False):
    """
    返回现在整数时间戳
    :param get13:
    :return:
    """
    if get13:
        return int(round(time.time() * 1000))
    else:
        return int(time.time())


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: file 
#       作者  : Qazse 
#       时间  : 2019/4/13
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------

def read_file_list(file_path, encoding='utf8'):
    """
    读取文件返回列表
    :param file: 文件目录
    :return:
    """
    file_list = []
    files = open(file_path, 'r', encoding=encoding)
    for file in files:
        file_list.append(file.strip())
    files.close()
    return file_list


def write_sql(sql, file_path='sql.sql', encoding='utf8',mode='a+'):
    """
    写sql
    :param sql: sql语句
    :param file_path: 写到目录
    :return:
    """
    with open(file_path, mode, encoding=encoding) as f:
        sql =str(sql)
        if not ';' in sql:
            sql = sql + ';'
        if not '\n' in sql:
            sql = sql + '\n'
        f.write(sql)

def write_text(text, file_path='text.txt', encoding='utf8',mode='a+'):
    """
    写text
    :param text:
    :param file_path: 写到目录
    :return:
    """
    with open(file_path, mode, encoding=encoding) as f:
        text = str(text)
        if not '\n' in text and not 'b' in mode:
            text = text + '\n'
        f.write(text)


def write_file(data, file_path='text.txt',mode='wb+'):
    """
    写文件
    :param data:二进制数据
    :param file_path: 写到目录
    :return:
    """
    with open(file_path, mode) as f:
        f.write(data)

def mkdir(path):
    """
    创建目录
    :param path:
    :return:
    """
    import os
    try:
        os.makedirs(path)
    except:
        pass


def get_md5(bit):
    """
    获取md5
    :param bit:
    :return:
    """
    from hashlib import md5

    return md5(bit).hexdigest()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: log 
#       作者  : Qazse 
#       时间  : 2019/4/13
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------

# logging.disable(logging.CRITICAL)   # 禁止输出日志
import logging
import os
import sys as _sys

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
_sys.path.append(BASE_DIR)

def log(logger_name='log-log', log_file=os.path.join(BASE_DIR, 'log', 'log.log'), level=logging.DEBUG,
        console_level=logging.DEBUG):
    try:
        os.mkdir('log')
    except:
        pass
    logger = logging.getLogger(logger_name)
    logger.setLevel(level)  # 添加日志等级

    # 创建控制台 console handler
    ch = logging.StreamHandler()
    # 设置控制台输出时的日志等级
    ch.setLevel(console_level)

    # 创建文件 handler
    fh = logging.FileHandler(filename=log_file, encoding='utf-8')
    # 设置写入文件的日志等级
    fh.setLevel(logging.DEBUG)
    # 创建 formatter
    formatter = logging.Formatter('%(asctime)s %(filename)s [line:%(lineno)d] %(name)s %(levelname)s %(message)s')

    # 添加formatter
    ch.setFormatter(formatter)
    fh.setFormatter(formatter)

    # 把ch fh 添加到logger
    logger.addHandler(ch)
    logger.addHandler(fh)

    return logger


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: spiders 
#       作者  : Qazse 
#       时间  : 2019/4/13
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------
import qazse


def ask_get(resp_text,url, question_title_css='', question_content_css='', answers_css='', remove_keyword='', type='auto',
            coding=None):
    """
    问答提取工具
    :param resp_text:返回的内容
    :param question_title_css: 例如.question-name
    :param question_content_css:
    :param answers_css: list css样式
    :param remove_keyword: list
    :param type: 类型 baidu wukong zhihu auto
    :param coding:编码类型
    :return: list
    """
    from bs4 import BeautifulSoup

    wukong_question_title = '.question-name'
    wukong_question_content = '.question-text'
    wukong_answers = ['.answer-text']
    wukong_remove = ['分享', '举报', '展开全部', '\d+评论', '评论']

    baidu_question_title = '.ask-title'
    baidu_question_content = '.line.mt-5.q-content'
    baidu_answers = ['.line.content']
    baidu_remove = ['分享', '举报', '展开全部', '\d+评论', '评论']

    zhihu_question_title = '.QuestionHeader-title'
    zhihu_question_content = '.RichText.ztext'
    zhihu_answers = ['.RichContent-inner']
    zhihu_remove = ['分享', '举报', '展开全部', '\d+评论', '评论','谢邀','邀请']

    if type == 'auto':
        if 'baidu' in url:
            type = 'baidu'
        elif 'wukong' in url:
            type = 'wukong'
        elif 'zhidao.baidu.com' in url:
            type = 'zhihu'

    if type == 'baidu':
        question_title_css = baidu_question_title
        question_content_css = baidu_question_content
        answers_css = baidu_answers
        remove_keyword = baidu_remove
    elif type == 'wukong':
        question_title_css = wukong_question_title
        question_content_css = wukong_question_content
        answers_css = wukong_answers
        remove_keyword = wukong_remove
    elif type == 'zhihu':
        question_title_css = zhihu_question_title
        question_content_css = zhihu_question_content
        answers_css = zhihu_answers
        remove_keyword = zhihu_remove
    soup = BeautifulSoup(resp_text, features="lxml")
    question_title = soup.select_one(question_title_css).text
    question_title = qazse.text.remove_n_r(question_title)
    question_content = soup.select_one(question_content_css).text if (soup.select_one(question_content_css)) else ''
    question_content = qazse.text.remove_n_r(question_content)
    question_md5 = qazse.text.md5_str(question_title + question_content)
    answers = list()
    for css in answers_css:
        answers = answers + soup.select(css)
    answers_list = []
    for index, answer in enumerate(answers):
        answer = qazse.text.remove_keyword(answer.text, remove_keyword)
        answer_md5 = qazse.text.md5_str(answer)
        answers_list.append({
            'content': answer,
            'md5': answer_md5
        })
    return {
        "question_title": question_title,
        "question_content": question_content,
        "question_md5": question_md5,
        "answers": answers_list,
        "answer_count": len(answers_list),
        "url": url,
        "type": type,
    }


def article_get(resp_text,url,title_css,content_css,remove_keyword=''):
    """
    提取文章
    :param resp_text:
    :param url:
    :param title_css:
    :param content_css:
    :param remove_keyword:
    :return:
    """
    from bs4 import BeautifulSoup
    soup = BeautifulSoup(resp_text, features="lxml")
    title = soup.select_one(title_css).text
    title = qazse.text.remove_n_r(title)
    title = qazse.text.remove_keyword(title, remove_keyword)
    content = soup.select_one(content_css)
    content = qazse.text.remove_n_r(content)
    content = qazse.text.remove_keyword(content, remove_keyword)
    md5 = qazse.text.md5_str(title + content)
    return {
        "title": title,
        "content": content,
        "md5": md5,
        "url": url,
    }
    pass

def browser(type = 1):
    """
    启动一个浏览器
    :param type: 1 chrome  2 firefox
    :return:
    """
    from selenium import webdriver
    if type == 1:
        b = webdriver.Chrome()
    elif type ==2:
        b = webdriver.Firefox()
    else:
        print('游览器类型不正确 1 chrome  2 firefox')
        return ''
    return b

def check_ban(page_source):
    """
    检查当前是否被禁止
    :param page_source:
    :return:
    """
    if '验证' in page_source:
        return True
    elif '异常' in page_source:
        return True
    else:
        return False

def check_ban_browser(browser,type = 1):
    """
    检测浏览器是否被禁止，如果被禁止将重启浏览器
    :param browser:
    :param method: 重启完毕要执行的函数
    :param type
    :return:
    """
    from selenium import webdriver
    if check_ban(browser.page_source):
        url = browser.back()
        url = browser.current_url
        browser.quit()
        if type == 1:
            browser = webdriver.Chrome()
        elif type ==2:
            browser = webdriver.Firefox()
        browser.get(url)
        browser.implicitly_wait(30)
        return browser,False
    else:
        return browser,True

def sogou_spider(keywords):
    """
    搜狗微信爬虫
    会激活一个google浏览器 环境尚未自动化配置 //todo
    2019年5月6日 16:53:25
    :param keywords:关键词们
    :return:
    """
    from bs4 import BeautifulSoup
    data = []
    b = browser()
    for keyword in keywords:
        def input_keyword(keyword,b):
            b.maximize_window()
            b.get('https://weixin.sogou.com/')
            b.implicitly_wait(30)
            b.find_element_by_class_name('query').clear()
            b.find_element_by_class_name('query').send_keys(keyword)
            b.find_element_by_class_name('swz').click()

        input_keyword(keyword,b)
        b,status = check_ban_browser(b)
        print(b.current_url)
        while status and b.current_url == 'https://weixin.sogou.com/':
            input_keyword(keyword,b)

        while '下一页' in b.page_source:
            b, status = check_ban_browser(b)
            while status and b.current_url == 'https://weixin.sogou.com/':
                input_keyword(keyword, b)

            b.implicitly_wait(30)
            soup = BeautifulSoup(b.page_source,'lxml')
            urls = soup.select('.txt-box h3 a')
            for url in urls:
                data.append({
                    'keyword':keyword,
                    'title':url.text,
                    'url':url['data-share']
                })
            b.find_element_by_link_text('下一页').click()
    b.quit()
    return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: seo 
#       作者  : Qazse 
#       时间  : 2019/4/13
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------
import requests


def baidu_weight(text):
    """
    获取百度权重
    :param text:
    :return:
    """
    from bs4 import BeautifulSoup
    import difflib

    url = "https://www.baidu.com/s"
    querystring = {"wd": text}
    headers = {
        'accept': "application/json, text/javascript",
        'accept-encoding': "gzip, deflate, br",
        'accept-language': "zh-CN,zh;q=0.9",
        'content-type': "application/x-www-form-urlencoded",
        'user-agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36",
        'x-requested-with': "XMLHttpRequest",
        'cache-control': "no-cache",
    }
    response = requests.request("GET", url, headers=headers, params=querystring)
    html = response.text
    soup = BeautifulSoup(html, 'html.parser')
    max_index = 0
    h3_list = soup.find_all('em')
    for h3 in h3_list:
        s1 = h3.text
        index = difflib.SequenceMatcher(None, text, s1).quick_ratio()
        if max_index <= index:
            max_index = index
    return max_index



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: requser 
#       作者  : Qazse 
#       时间  : 2019/4/13
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------

import requests
import qazse


def useragent(device=0):
    """
    返回一个头
    :param device:
    :return:
    """
    headers = {
        'user-agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/72.0.3626.109 Safari/537.36",
        'cache-control': "no-cache",
    }
    return headers


def request_get(url, params=None, proxy=None, encoding=None, **kwargs):
    """
    get参数
    :param url:
    :param params:
    :param proxy:
    :param kwargs:
    :return:
    """
    from requests.adapters import HTTPAdapter
    if proxy:
        proxies = {
            'http': 'http://%s' % proxy,
            'https': 'https://%s' % proxy
        }
    else:
        proxies = {
            'http': None,
            'https': None
        }
    s = requests.Session()
    s.mount('http://', HTTPAdapter(max_retries=3))
    s.mount('https://', HTTPAdapter(max_retries=3))
    try:
        response = requests.get(url, params, proxies=proxies, headers=useragent(), **kwargs)
        if encoding:
            response.encoding = encoding
        return response
    except requests.exceptions.RequestException as e:
        print(e)
        return None


def max_request_get(urls_info, max_pool=100, config=None):
    """
    多线程读HTTP
    :param urls: dict
        {
            url:'http://xxx', 地址
            data:'xxx',   数据
            proxy:'data', 代理
            method:"GET", 方法
            headers:"",   头
            save_name:"", 保存文件名
            save_type:1   保存方法 1 文件 2 mongodb
        }
    :param max_pool: 最大线程
    :param config:
        {
            directory:"path", 保存路径
            save_db:"",       保存mongodb 对象
            save_status_200:true, 只保存http=200
            show_log:true       显示日志
        }
    :return:
    """
    directory = config.get('directory', 'data')
    save_status_200 = config.get('save_status_200', True)
    show_log = config.get('show_log', True)
    save_db = config.get('save_db',qazse.db.mongodb_db())

    import aiohttp, asyncio
    from qazse import file

    file.mkdir(directory)

    async def main(pool):  # aiohttp必须放在异步函数中使用
        tasks = []
        sem = asyncio.Semaphore(pool)  # 限制同时请求的数量
        for url_info in urls_info:
            tasks.append(control_sem(sem, url_info))
        await asyncio.wait(tasks)

    async def control_sem(sem, url_info):  # 限制信号量
        async with sem:
            await fetch(url_info)

    async def fetch(url_info):
        # 解析配置
        url = url_info.get('url')
        data = url_info.get('data', '')
        method = url_info.get('method', 'GET')
        proxy = url_info.get('proxy', '')
        headers = url_info.get('headers', useragent())
        save_name = url_info.get('save_name', '%s.html' % url)
        save_type = url_info.get('save_type', 1)

        async with aiohttp.request(method, url_info.get('url'), data=data, headers=headers, proxy=proxy) as resp:
            if show_log:
                print('Download', url, method, proxy, resp.status)

            if save_status_200 and resp.status == 200:
                if save_type == 1:
                    file.write_file(await resp.read(), file_path=directory + '/' + save_name)
                elif save_type ==2:
                    save_db.save(await resp.read())
            elif not save_status_200:
                if save_type == 1:
                    file.write_file(await resp.read(), file_path=directory + '/' + save_name)
                elif save_type ==2:
                    save_db.save(await resp.read())

    loop = asyncio.get_event_loop()
    loop.run_until_complete(main(pool=max_pool))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: setup.py 
#       作者  : Qazse 
#       时间  : 2019/4/13
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------

from setuptools import setup, find_packages

setup(
    name='qazse',
    version="0.6.3",
    keywords=("pip", "qazse", "QazseWong"),
    packages=find_packages(),
    author='QazseWong',
    author_email='w@qiiing.com',
    url='http://w.qiiing.com',
    license="MIT Licence",
    description='自己用的工具包',
    long_description="自己python项目常用得工具包",
    include_package_data=True,
    platforms="any",
    install_requires=["redis","requests","aiohttp","asyncio","bs4","selenium"]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: wq 
#       作者  : Qazse 
#       时间  : 2019/4/13
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------


def write_text_clipboard(text):
    """
    写文本到剪切板
    :param text:
    :return:
    """
    import pyperclip
    pyperclip.copy(text)


def copy_text_from_clipboard():
    """
    读取剪切板的文本
    :return:
    """
    import pyperclip
    return pyperclip.paste()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------
#       项目名: qazse   
#       文件名: text 
#       作者  : Qazse 
#       时间  : 2019/4/13
#       主页  : http://qiiing.com 
#       功能  :
# ---------------------------------------------------
import re


def remove_emoji(text):
    """
    删除emoji
    :param text:
    :return:
    """
    try:
        data = re.compile(u'[\U00010000-\U0010ffff]')
    except re.error:
        data = re.compile(u'[\uD800-\uDBFF][\uDC00-\uDFFF]')
    return data.sub(u'', text)


def md5_str(text):
    """
    获取MD5
    :param text:
    :return:
    """
    from hashlib import md5

    return md5(str(text).encode()).hexdigest()

def remove_keyword(text,keywords):
    """
    删除指定关键字,支持正则
    :param text:
    :param keyword:
    :return:
    """
    import re
    for keyword in keywords:
        # text = str(text).replace(keyword,'')
        text = re.sub(keyword,'',text)
    return remove_n_r(text)

def remove_url(text):
    """
    删除网页中的Url
    :param text:
    :return:
    """
    return re.sub(r'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+','',text)


def remove_n_r(text):
    """
    删除空格。换行符
    :param text:
    :return:
    """
    return str(text).strip()

def remove_html_tag(html,tags = None):
    """
    删除HTML中的标签
    :param html:
    :param tags:
    :return:
    """
    from w3lib.html import remove_tags
    if tags:
        return remove_tags(html,which_ones=(tags))
    else:
        return remove_tags(html,keep=())

def json_dumps(json_data):
    """
    还原json文本
    :param json_data:
    :return:
    """
    import json
    return json.dumps(json_data)


if __name__ == '__main__':
    text = '<p>非常好看，穿出设计师设计出来的那种范来了，就完美了。</p><p><img src="49839281888"/></p>'
    print(remove_url(text))